# encoding: utf-8
# IMIO <support@imio.be>
#

"""
Batching utilities methods to do a process in multiple passes (divide a for loop).
Idea: a batch number, a commit number and a loop number are considered

1) we get a stored dictionary containing the treated keys (using load_pickle function)
2) if the key is already in the dictionary, we skip it (continue). Otherwise, we increase the loop count
3) if the treated items number is >= batch number, we exit the for loop, do a commit and dump the dictionary
4) otherwise, we store the corresponding key in the dictionary
5) when the current loop number is a multiple of the commit number, we do a commit and dump the dictionary
6) when the for loop is globally finished, we do a commit and dump the dictionary
7) when all the items are treated, we can delete the dictionary file

See `loop_process` function in `test_batching.py` file for a complete example.
"""

from imio.pyutils.batching import batch_delete_files  # noqa: F401
from imio.pyutils.system import dump_pickle
from imio.pyutils.system import dump_var
from imio.pyutils.system import hashed_filename
from imio.pyutils.system import load_pickle

import logging
import os
import transaction


logger = logging.getLogger("imio.helpers")


# 1) we get a stored dictionary containing the treated keys (using load_pickle function)
def batch_get_keys(infile, loop_length=0, a_set=None, add_files=None):
    """Returns the stored batched keys from the file.
    Must be used like this, before the loop:
    batch_keys, config = batch_get_keys(infile, batch_number, commit_number)

    :param infile: file name where the set is stored
    :param loop_length: the loop length number
    :param a_set: a given data structure to get the stored keys
    :param add_files: a list of additional files to consider when deleting files
    :return: 2 parameters: 1) a_set fulled with pickled data,
    2) a config dict {'bn': batch_number, 'bl': batch_last, 'cn': commit_number, 'll': loop_length, 'lc': loop_count,
                      'pf': infile, 'cf': config_file, 'kc': keys_count, 'lk': last_key, 'ldk': last_dump_key,
                      'fr'; first_run_bool, 'af': add_files}
    """
    infile = os.path.abspath(infile)
    commit_number = int(os.getenv("COMMIT", "0"))
    batch_number = int(os.getenv("BATCH", "0"))
    batch_last = bool(int(os.getenv("BATCH_LAST", "0")))
    if not add_files:
        add_files = []
    if not batch_number:
        return None, {
            "bn": batch_number,
            "bl": batch_last,
            "cn": commit_number,
            "ll": loop_length,
            "lc": 0,
            "pf": infile,
            "cf": None,
            "kc": 0,
            "fr": False,
            "af": add_files,
        }
    if not infile.endswith(".pkl"):
        raise Exception("The given file '{}' must be a pickle file ending with '.pkl'".format(infile))
    if a_set is None:
        a_set = set()
    load_pickle(infile, a_set)
    dic_file = infile.replace(".pkl", "_config.txt")
    first_run = not os.path.exists(dic_file)
    config = {
        "bn": batch_number,
        "bl": batch_last,
        "cn": commit_number,
        "ll": loop_length,
        "lc": 0,
        "pf": infile,
        "cf": dic_file,
        "kc": len(a_set),
        "fr": first_run,
        "af": add_files,
    }
    dump_var(dic_file, config)
    return a_set, config


# 2) if the key is already in the dictionary, we skip it (continue). Otherwise, we increase the loop count
def batch_skip_key(key, batch_keys, config):
    """Returns True if the key is already in the batch_keys.
    Must be used like this, at the beginning of the loop:
    if batch_skip_key(key, batch_keys):
        continue

    :param key: the hashable key of the current item
    :param batch_keys: the treated keys set
    :param config: a config dict {'bn': batch_number, 'bl': batch_last, 'cn': commit_number, 'll': loop_length,
                                  'lc': loop_count, 'pf': infile, 'cf': config_file, 'kc': keys_count, 'lk': last_key,
                                  'ldk': last_dump_key, 'fr'; first_run_bool, 'af': add_files}
    :return: True if a "continue" must be done. False otherwise.
    """
    if batch_keys is None:
        return False
    if key not in batch_keys:
        config["lc"] += 1
        return False
    return True


# 3) if the treated items number is higher than the batch number, we exit the loop, do a commit and dump the dictionary
# 4) otherwise, we store the corresponding key in the dictionary
# 5) when the current loop number is a multiple of the commit number, we do a commit and dump the dictionary
def batch_handle_key(key, batch_keys, config):
    """Returns True if the loop must be exited.
    Must be used like this, in the end of the loop:
    if batch_handle_key(key, batch_keys, config):
        break

    :param key: the hashable key of the current item
    :param batch_keys: the treated keys set
    :param config: a config dict {'bn': batch_number, 'bl': batch_last, 'cn': commit_number, 'll': loop_length,
                                  'lc': loop_count, 'pf': infile, 'cf': config_file, 'kc': keys_count, 'lk': last_key,
                                  'ldk': last_dump_key, 'fr'; first_run_bool, 'af': add_files}
    :return: True if the loop must be exited. False otherwise.
    """
    if batch_keys is None:
        return False
    batch_keys.add(key)
    config["lk"] = key
    # stopping batch ?
    if config["lc"] >= config["bn"]:
        if config["cn"]:
            transaction.commit()
        config["ldk"] = key
        config["kc"] = len(batch_keys)
        dump_pickle(config["pf"], batch_keys)
        dump_var(config["cf"], config)
        logger.info("BATCHED %s / %s, already done %s", config["lc"], config["ll"], config["kc"])
        if config["bl"] and not batch_globally_finished(batch_keys, config):
            logger.error("BATCHING MAYBE STOPPED TOO EARLY: %s / %s", config["kc"], config["ll"])
        return True
    # commit time ?
    if config["cn"] and config["lc"] % config["cn"] == 0:
        transaction.commit()
        config["ldk"] = key
        config["kc"] = len(batch_keys)
        dump_pickle(config["pf"], batch_keys)
        dump_var(config["cf"], config)
    return False


# 6) when the loop is globally finished, we do a commit and dump the dictionary
def batch_loop_else(batch_keys, config):
    """Does a commit and dump the dictionary when the loop is globally finished.
    Must be used like this, in the else part of the for loop:
    batch_loop_else(batch_keys, config)

    :param batch_keys: the treated keys set
    :param config: a config dict {'bn': batch_number, 'bl': batch_last, 'cn': commit_number, 'll': loop_length,
                                  'lc': loop_count, 'pf': infile, 'cf': config_file, 'kc': keys_count, 'lk': last_key,
                                  'ldk': last_dump_key, 'fr'; first_run_bool, 'af': add_files}
    """
    if batch_keys is None:
        return
    last_key = config.get("lk")
    # avoid if nothing was done or already done on last key
    if last_key is None or (config.get("ldk") is not None and config["ldk"] == last_key):
        return
    logger.info("BATCHED %s / %s, already done %s", config["lc"], config["ll"], len(batch_keys))
    if config["cn"]:
        transaction.commit()
    config["kc"] = len(batch_keys)
    dump_pickle(config["pf"], batch_keys)
    dump_var(config["cf"], config)
    if config["bl"] and not batch_globally_finished(batch_keys, config):
        logger.error("BATCHING MAYBE STOPPED TOO EARLY: %s / %s", config["kc"], config["ll"])


# 7) when all the items are treated, we can delete the dictionary file
# we use now batch_delete_files from imio.pyutils


def batch_globally_finished(batch_keys, config):
    """Is the loop globally finished?

    :param batch_keys: the treated keys set
    :param config: a config dict {'bn': batch_number, 'bl': batch_last, 'cn': commit_number, 'll': loop_length,
                                  'lc': loop_count, 'pf': infile, 'cf': config_file, 'kc': keys_count, 'lk': last_key,
                                  'ldk': last_dump_key, 'fr'; first_run_bool, 'af': add_files}
    :return: True if the loop is globally finished. False otherwise.
    """
    # if not batch_keys:
    #     return True
    if not config["bn"]:  # no batching
        return True
    if config["lc"] == 0:  # nothing treated
        return True
    elif config["fr"]:  # first run with results
        return len(batch_keys) >= config["ll"]
    elif config["bl"]:
        return True
    return False


def batch_hashed_filename(filename, to_hash=(), add_dir=True):
    """Returns a hashed filename.

    :param filename: the filename
    :param to_hash: a list of values to hash
    :param add_dir: boolean to know if we must prefix with the INSTANCE_HOME directory
    :return: the hashed filename
    """
    pklfile = hashed_filename(filename, "_".join(map(repr, to_hash)))
    if add_dir:
        pklfile = os.path.join(os.getenv("INSTANCE_HOME", "."), pklfile)
    return pklfile


def can_delete_batch_files(batch_keys, config):
    """Returns True if the batch config files can be deleted.

    :param batch_keys: the treated keys set
    :param config: a config dict {'bn': batch_number, 'bl': batch_last, 'cn': commit_number, 'll': loop_length,
                                  'lc': loop_count, 'pf': infile, 'cf': config_file, 'kc': keys_count, 'lk': last_key,
                                  'ldk': last_dump_key, 'fr'; first_run_bool, 'af': add_files}
    :return: boolean
    """
    if config["fr"] and os.getenv("IU_RUN1", "0") == "1":  # if first run by imio.updates, the config file is needed.
        return False
    return batch_globally_finished(batch_keys, config)



# -*- coding: utf-8 -*-

from Acquisition import aq_inner
from imio.helpers import HAS_PLONE_4
from imio.helpers.interfaces import IListContainedDexterityObjectsForDisplay
from plone import api
from plone.app.caching.browser.controlpanel import RAMCache
from plone.batching import Batch
from plone.dexterity.browser.view import DefaultView
from plone.registry.interfaces import IRegistry
from Products.Five import BrowserView
from zope.component import getMultiAdapter
from zope.component import getUtility


if HAS_PLONE_4:
    from plone.app.content.browser.foldercontents import FolderContentsBrowserView
    from plone.app.content.browser.foldercontents import FolderContentsTable
    from plone.app.content.browser.foldercontents import FolderContentsView
else:
    FolderContentsBrowserView = object
    FolderContentsTable = object
    FolderContentsView = object


class ContainerView(DefaultView):
    """ """

    collapse_all_fields = False
    collapse_all_fields_onload = False
    fields_to_collapse = []  # not yet used
    fields_to_collapse_onload = []  # not yet used


class ContainerFolderListingView(BrowserView):
    """
    This manage the elements listed on the view of a dexterity container
    """

    def __init__(self, context, request):
        self.context = context
        self.request = request
        portal_state = getMultiAdapter((self.context, self.request), name=u"plone_portal_state")
        self.portal = portal_state.portal()

    def listRenderedContainedElements(self, portal_types=(), widgets_to_render=(), b_size=30, b_start=0):
        """
        Get the contained elements, rendered for display.
        If p_portal_types is specified, only return elements having the required portal_type.
        If p_widgets_to_render is specified, only render given fields/widgets.
        """
        result = IListContainedDexterityObjectsForDisplay(self.context).listContainedObjects(
            portal_types, widgets_to_render, b_start=b_start, b_size=b_size
        )
        batch = Batch(result, b_size, b_start, orphan=1)
        return batch

    def author(self, author_id):
        """
        Return fullname of given p_author_id
        """
        membership = api.portal.get_tool("portal_membership")
        memberInfos = membership.getMemberInfo(author_id)
        return memberInfos and memberInfos["fullname"] or author_id

    def authorname(self):
        author = self.author()
        return author and author["fullname"] or self.creator()

    def update_table(self):
        return self.context.restrictedTraverse("@@imio-folder-listing-table").index()


class IMIORAMCache(RAMCache):
    """ """

    def update(self):
        super(IMIORAMCache, self).update()
        self.stats = self.ramCache.getStatistics()


class IMIOFolderContentsTable(FolderContentsTable):
    """Overrided so @@folder_contents work on a DashboardCollection."""

    def contentsMethod(self):
        context = aq_inner(self.context)

        # use DashboardCollection.brains_results if available
        contentsMethod = None
        if hasattr(context.__class__, "brains_results"):
            contentsMethod = context.brains_results
        return contentsMethod or super(IMIOFolderContentsTable, self).contentsMethod()


class IMIOFolderContentsBrowserView(FolderContentsBrowserView):
    table = IMIOFolderContentsTable


class IMIOFolderContentsView(FolderContentsView):
    def contents_table(self):
        # override to use IMIOFolderContentsTable instead FolderContentsTable
        table = IMIOFolderContentsTable(aq_inner(self.context), self.request)
        return table.render()


class RegistryIconsView(BrowserView):
    """View to list icons from the registry."""

    def get_icons(self):
        registry = getUtility(IRegistry)
        icons = []
        for key in registry.records.keys():
            value = api.portal.get_registry_record(key)
            if isinstance(value, str) and value.endswith((".svg", ".png", ".jpg", ".jpeg", ".gif")):
                icons.append({"key": key, "path": value})
        return sorted(icons, key=lambda x: x["key"])



from imio.helpers.utils import is_valid_json
from Products.CMFPlone.utils import safe_unicode
from zope.interface import Invalid

import unittest


class TestUtils(unittest.TestCase):
    def test_valid_json(self):
        malicious_input = safe_unicode(""" ""};<script>alert('I'm malicious')</script> """)
        good_input = '[".my-class"]'
        with self.assertRaises(Invalid):
            is_valid_json(malicious_input)
        self.assertTrue(is_valid_json(good_input))



# -*- coding: utf-8 -*-
"""Test fancytree."""
from imio.helpers.fancytree.views import BaseRenderFancyTree
from imio.helpers.testing import IntegrationTestCase
from plone import api

import json


class RenderFancyTreeExample(BaseRenderFancyTree):

    """Fancy tree view with dummy get_query implementation for tests purpose."""

    def index(self):
        return "index"

    def get_query(self):
        return {
            "path": {"query": self.root_path, "depth": -1},
            "portal_type": (
                "Folder",
                "Document",
            ),
        }

    def redirect_url(self, uid):
        return uid


class TestBaseRenderFancyTree(IntegrationTestCase):

    """Test BaseRenderFancyTree view."""

    def test_label(self):
        view = BaseRenderFancyTree(self.portal, self.request)
        self.assertEqual(view.label(), "Plone site")

    def test_redirect_url(self):
        view = BaseRenderFancyTree(self.portal, self.request)
        with self.assertRaises(NotImplementedError):
            view.redirect_url("some_uid")

    def test_get_query(self):
        view = BaseRenderFancyTree(self.portal, self.request)
        with self.assertRaises(NotImplementedError):
            view.get_query()

    def test_call(self):
        view = RenderFancyTreeExample(self.portal, self.request)
        self.assertEqual(view(), "index")

        request = self.request
        request.method = "POST"
        request["uid"] = "myuid"
        self.assertEqual(view(), "")
        self.assertEqual(view.request.response.headers["location"], "myuid")

    def test_get_data(self):
        """Test get_data method."""
        view = RenderFancyTreeExample(self.portal, self.request)
        self.assertEqual(view.get_data(), "[]")

        # add some folders and documents
        myfolder = api.content.create(container=self.portal, type="Folder", id="myfolder", title="My folder")
        mydoc1 = api.content.create(container=myfolder, type="Document", id="mydoc1", title="My document")
        mydoc2 = api.content.create(container=myfolder, type="Document", id="mydoc2", title="My other document")
        output = view.get_data()
        expected = json.dumps(
            [
                {
                    "key": myfolder.UID(),
                    "folder": True,
                    "title": "My folder",
                    "children": [
                        {
                            "key": mydoc1.UID(),
                            "folder": False,
                            "title": "My document",
                        },
                        {
                            "key": mydoc2.UID(),
                            "folder": False,
                            "title": "My other document",
                        },
                    ],
                }
            ]
        )
        expected = sorted(json.loads(expected))
        output = sorted(json.loads(output))
        self.assertEqual(output, expected)



# -*- coding: utf-8 -*-
"""Installer for the imio.helpers package."""

from setuptools import find_packages
from setuptools import setup


long_description = open("README.rst").read() + "\n" + open("CHANGES.rst").read() + "\n"

setup(
    name="imio.helpers",
    version="1.3.1",
    description="Various helper methods for development.",
    long_description=long_description,
    # Get more from http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 4.3",
        "Framework :: Plone :: 6.0",
        "Framework :: Plone :: 6.1",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
    ],
    keywords="plone helpers utils dev imio",
    author="Simon Delcourt",
    author_email="simon.delcourt@imio.be",
    url="https://github.com/imio/imio.helpers",
    download_url="https://pypi.org/project/imio.helpers",
    license="GPL",
    packages=find_packages("src", exclude=["ez_setup"]),
    namespace_packages=["imio"],
    package_dir={"": "src"},
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "collective.fingerpointing",
        "collective.monkeypatcher",
        "future>=0.18.2",
        "imio.pyutils>=1.1.1",
        "natsort",
        "pathlib2;python_version<'3'",
        "plone.api>1.9.1",
        "plone.app.intid",
        "plone.app.relationfield",
        "plone.dexterity",
        "setuptools",
        "six",
        "Plone",
        "z3c.unconfigure",
    ],
    extras_require={
        "test": [
            "collective.behavior.talcondition",
            "collective.MockMailHost",
            "plone.app.dexterity",
            "plone.app.testing",
        ],
        "pdf": [
            "PyPDF2",
            "reportlab",
        ],
        "lxml": [
            'cssselect<1.2; python_version == "2.7"',
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
    options={"bdist_wheel": {"universal": True}},
)



# -*- coding: utf-8 -*-

from DateTime import DateTime
from imio.helpers.date import formatDate
from imio.helpers.date import int2word
from imio.helpers.date import wordizeDate
from imio.helpers.testing import IntegrationTestCase

import six


class TestDateModule(IntegrationTestCase):
    """
    Test all helper methods of date module.
    """

    def test_wordizeDate(self):
        if six.PY3:
            self.assertEqual(wordizeDate(DateTime(12, 12, 12)), "douze décembre deux mille douze")
            self.assertEqual(
                wordizeDate(DateTime(12, 12, 12, 12, 12), long_format=True),
                "douze décembre deux mille douze à douze heures douze",
            )
            self.assertEqual(
                wordizeDate(DateTime(12, 12, 12, 12), long_format=True),
                "douze décembre deux mille douze à douze heures",
            )
            self.assertEqual(
                wordizeDate(DateTime(1, 1, 1, 0, 1), long_format=True), "premier janvier deux mille un à zéro heure un"
            )
            self.assertEqual(
                wordizeDate(DateTime(1, 1, 1, 1), long_format=True), "premier janvier deux mille un à une heure"
            )
        else:
            self.assertEqual(wordizeDate(DateTime(12, 12, 12)), "douze d\xc3\xa9cembre deux mille douze")
            self.assertEqual(
                wordizeDate(DateTime(12, 12, 12, 12, 12), long_format=True),
                "douze d\xc3\xa9cembre deux mille douze \xc3\xa0 douze heures douze",
            )
            self.assertEqual(
                wordizeDate(DateTime(12, 12, 12, 12), long_format=True),
                "douze d\xc3\xa9cembre deux mille douze \xc3\xa0 douze heures",
            )
            self.assertEqual(
                wordizeDate(DateTime(1, 1, 1, 0, 1), long_format=True),
                "premier janvier deux mille un \xc3\xa0 z\xc3\xa9ro heure un",
            )
            self.assertEqual(
                wordizeDate(DateTime(1, 1, 1, 1), long_format=True), "premier janvier deux mille un \xc3\xa0 une heure"
            )
        self.assertEqual(wordizeDate(DateTime(11, 11, 11)), "onze novembre deux mille onze")
        self.assertEqual(wordizeDate(DateTime(1989, 9, 19)), "dix-neuf septembre mille neuf cent quatre-vingt-neuf")
        self.assertEqual(wordizeDate(DateTime(1, 1, 1)), "premier janvier deux mille un")

    def test_formatDate(self):
        self.assertEqual(formatDate("now", month_name=False), DateTime().strftime("%d/%m/%Y"))
        self.assertEqual(
            formatDate(DateTime(2, 1, 1, 1, 1), month_name=True, long_format=True), u"1er janvier 2002 (01:01)"
        )
        self.assertEqual(
            formatDate(DateTime(1, 1, 1, 1, 1), month_name=True, long_format=True), u"1er janvier 2001 (01:01)"
        )

    def test_int2word(self):
        self.assertEqual(int2word(21), "vingt et un")
        self.assertEqual(int2word(100), "cent")
        self.assertEqual(int2word(220), "deux cent vingt")
        self.assertEqual(int2word(1000100), "un million cent")
        self.assertEqual(
            int2word(123456789876543212345678987654321),
            "cent vingt-trois octilliard quatre cent cinquante-six "
            "septilliard sept cent quatre-vingt-neuf sextilliard huit "
            "cent septante-six quintillard cinq cent quarante-trois "
            "quadrilliard deux cent douze trilliard trois cent quarante-cinq "
            "billiard six cent septante-huit milliard neuf cent quatre-vingt-sept "
            "million six cent cinquante-quatre mille trois cent vingt et un",
        )



# -*- coding: utf-8 -*-
from imio.helpers.testing import IntegrationTestCase
from imio.helpers.workflow import do_transitions
from imio.helpers.workflow import get_final_states
from imio.helpers.workflow import get_leading_transitions
from imio.helpers.workflow import get_state_infos
from imio.helpers.workflow import get_transitions
from imio.helpers.workflow import remove_state_transitions
from imio.helpers.workflow import update_role_mappings_for
from plone import api
from Products.CMFCore.permissions import View


class TestWorkflowModule(IntegrationTestCase):
    """
    Test all helper methods of workflow module.
    """

    def setUp(self):
        self.portal = self.layer["portal"]

    def test_do_transitions(self):
        obj = api.content.create(container=self.portal.folder, id="mydoc", type="Document")
        self.assertEqual(api.content.get_state(obj), "internal")
        # apply one transition
        do_transitions(obj, "submit")
        self.assertEqual(api.content.get_state(obj), "pending")
        # apply multiple transitions
        do_transitions(obj, ["publish_internally", "publish_externally"])
        self.assertEqual(api.content.get_state(obj), "external")
        do_transitions(obj, ["submit", "retract", "hide"])
        self.assertEqual(api.content.get_state(obj), "private")

    def test_get_state_infos(self):
        self.portal.portal_workflow.setChainForPortalTypes(("Document",), ("intranet_workflow",))
        obj = api.content.create(container=self.portal.folder, id="mydoc", title="My document", type="Document")
        self.assertDictEqual(get_state_infos(obj), {"state_title": u"Internal draft", "state_name": "internal"})
        do_transitions(obj, "submit")
        self.assertDictEqual(get_state_infos(obj), {"state_title": u"Pending review", "state_name": "pending"})

    def test_get_transitions(self):
        obj = api.content.create(container=self.portal.folder, id="mydoc", type="Document")
        sorted_obj_transitions = sorted(get_transitions(obj), key=str.lower)
        self.assertEqual(sorted_obj_transitions, ["hide", "publish_internally", "submit"])
        do_transitions(obj, "submit")
        sorted_obj_transitions = sorted(get_transitions(obj), key=str.lower)
        self.assertEqual(sorted_obj_transitions, ["publish_externally", "publish_internally", "reject", "retract"])
        do_transitions(obj, "publish_externally")
        self.assertEqual(get_transitions(obj), ["retract"])

    def test_remove_state_transitions(self):
        wf_tool = api.portal.get_tool("portal_workflow")
        wf = wf_tool["intranet_workflow"]
        self.assertFalse(remove_state_transitions("intranet_workflow", "unknown_state"))
        self.assertTupleEqual(wf.states["internal"].transitions, ("hide", "publish_internally", "submit"))
        self.assertFalse(remove_state_transitions("intranet_workflow", "internal"))  # nothing removed
        self.assertTrue(remove_state_transitions("intranet_workflow", "internal", ["hide", "submit"]))  # removed
        self.assertTupleEqual(wf.states["internal"].transitions, ("publish_internally",))
        # add duplicates to simulate bad manipulation
        wf.states["internal"].transitions = tuple(["publish_internally", "submit", "submit"])
        self.assertTupleEqual(wf.states["internal"].transitions, ("publish_internally", "submit", "submit"))
        self.assertTrue(remove_state_transitions("intranet_workflow", "internal"))
        self.assertTupleEqual(wf.states["internal"].transitions, ("publish_internally", "submit"))

    def test_get_leading_transitions(self):
        wf_tool = api.portal.get_tool("portal_workflow")
        wf = wf_tool["intranet_workflow"]
        self.assertEqual(get_leading_transitions(wf, None), [])
        self.assertEqual(get_leading_transitions(wf, "unknown"), [])
        self.assertEqual(get_leading_transitions(wf, "external"), [wf.transitions["publish_externally"]])
        # not_starting_with parameter will let ignore some transitions
        self.assertEqual(get_leading_transitions(wf, "external", not_starting_with="publish_"), [])
        self.assertEqual(
            get_leading_transitions(wf, "external", not_starting_with="suffix_"), [wf.transitions["publish_externally"]]
        )

    def test_update_role_mappings_for(self):
        obj = api.content.create(container=self.portal.folder, id="mydoc", type="Document")
        wf_tool = api.portal.get_tool("portal_workflow")
        self.assertEqual(wf_tool.getInfoFor(obj, "review_state"), "internal")
        wf = wf_tool.getWorkflowsFor(obj)[0]
        wf.states.internal.permission_roles[View] = ("Manager",)
        self.assertTrue(update_role_mappings_for(obj))
        # this is already tested in
        # collective.eeafaceted.batchactions.tests.test_forms.test_update_wf_role_mappings_action

    def test_get_final_states(self):
        wf_tool = api.portal.get_tool("portal_workflow")
        wf = wf_tool.getWorkflowById("intranet_folder_workflow")
        self.assertEqual(get_final_states(wf), [])
        wf = wf_tool.getWorkflowById("one_state_workflow")
        self.assertEqual(get_final_states(wf), ["published"])
        wf = wf_tool.getWorkflowById("intranet_folder_workflow")
        self.assertEqual(get_final_states(wf, ignored_transition_ids=["hide"]), ["internal"])
        wf = wf_tool.getWorkflowById("plone_workflow")
        self.assertEqual(get_final_states(wf, ignored_transition_ids=["reject", "retract"]), ["published"])



# -*- coding: utf-8 -*-

from AccessControl.SecurityManagement import getSecurityManager
from AccessControl.SecurityManagement import newSecurityManager
from App.config import getConfiguration
from collective.fingerpointing.config import AUDIT_MESSAGE
from collective.fingerpointing.logger import log_info
from collective.fingerpointing.utils import get_request_information
from itertools import chain
from plone import api
from plone.api.validation import at_least_one_of
from plone.api.validation import mutually_exclusive_parameters
from Products.CMFPlone.utils import safe_unicode
from random import choice
from random import sample
from random import seed
from six.moves import range
from time import time
from zope.component import getMultiAdapter
from zope.globalrequest import getRequest
from zope.globalrequest import setRequest

import inspect
import logging
import os
import string
import Zope2


logger = logging.getLogger("imio.helpers")


def is_develop_environment():
    """
    Test if the environment variable named ENV is added by the buildout and get the value
    """
    TRUISMS = ["yes", "y", "true", "on"]
    var = os.getenv("IS_DEV_ENV", "false")

    if var.lower() in TRUISMS:
        logger.info("IS_DEV_ENV is deprecated, please use ENV variable.")
        return True
    elif get_environment() == "dev":
        return True
    else:
        return False


def get_environment():
    """
    Get value of ENV environment variable.
    Value should be : dev, staging, preprod or prod.
    """
    env = os.getenv("ENV", "prod")
    return env.lower()


def generate_password(length=10, digits=3, upper=2, lower=1, special=1, readable=True):
    """
    Create a random password with the specified length and minimum numbers
    of digit, upper and lower case letters, special characters.
    """
    seed(time())
    # string.punctuation contains !"#$%&\'()*+,-./:;<=>?@[\\]^_`{|}~
    specials = "!#$%&*+-<=>?@"
    lowercase = "abcdefghijklmnopqrstuvwxyz"
    uppercase = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    letters = "{0:s}".format(lowercase)

    password = list(
        chain(
            (choice(uppercase) for _ in range(upper)),
            (choice(lowercase) for _ in range(lower)),
            (choice(letters) for _ in range((length - digits - upper - lower - special))),
            (choice(specials) for _ in range(special)),
            (choice(string.digits) for _ in range(digits)),
        )
    )
    if not readable:
        password = sample(password, len(password))
    return "".join(password)


def setup_app(app, username="admin", logger=None):
    # import here to avoid weird init making Data.fs not useable
    from Testing.makerequest import makerequest

    acl_users = app.acl_users
    user = acl_users.getUser(username)
    if user:
        user = user.__of__(acl_users)
        newSecurityManager(None, user)
    elif logger:
        logger.error("Cannot find zope user '%s'" % username)
    app = makerequest(app)
    # support plone.subrequest
    app.REQUEST["PARENTS"] = [app]
    setRequest(app.REQUEST)
    return user


def setup_logger(level=20):
    """
    When running "bin/instance run ...", the logger level is 30 (warn).
    It is possible to set it to 20 (info) or 10 (debug).
    """
    log = logging.getLogger()
    log.setLevel(level)
    for handler in logging.root.handlers:
        if handler.level and handler.level != level and handler.formatter is not None:
            handler.level = level
            break


def fplog(action, extras):
    """collective.fingerpointing add log message."""
    user, ip = get_request_information()
    log_info(AUDIT_MESSAGE.format(user, ip, action, extras))


@mutually_exclusive_parameters("email", "fullname", "userid")
@at_least_one_of("email", "fullname", "userid")
def get_user_from_criteria(context, email=None, fullname=None, userid=None):
    """
    :param context: context, with request as context.REQUEST
    :param email: part of user email
    :param fullname: part of user fullname
    :param userid: part of userid
    :return: list of dict describing users
             [{'description': u'Bob Smith', 'title': u'Bob Smith', 'principal_type': 'user', 'userid': 'bsm',
               'email': 'bsm@mail.com', 'pluginid': 'mutable_properties', 'login': 'bsm', 'id': 'bsm'}]
             [{'dn': 'CN=Luc.Oil,OU=Service_Social,OU=Utilisateurs,DC=cpas,DC=local', 'sAMAccountName':
               'luc.oil', 'title': 'luc.oil', 'editurl': 'ldap-plugin/acl_users/manage_userrecords?user_dn=...'
               'principal_type': 'user', 'userid': 'luc.oil', 'pluginid': 'ldap-plugin', 'sn': '', 'login': 'luc.oil',
               'id': 'luc.oil', 'cn': '', 'description': 'Luc.Oil', 'email': 'luc.oil@xx.com'}]
    """
    # TODO check if cache is necessary ???
    hunter = getMultiAdapter((context, context.REQUEST), name="pas_search")
    criteria = {}
    if email:
        criteria["email"] = email
    if fullname:
        criteria["fullname"] = fullname
    if userid:
        criteria["id"] = userid
    res = hunter.searchUsers(**criteria)
    for dic in res:
        if "email" not in dic or "description" not in dic:  # ldap
            member = api.user.get(userid=dic["userid"])
            dic["email"] = member.getProperty("email", default="")  # following plonepas.plugins.property.enumerateUsers
            dic["description"] = safe_unicode(member.getProperty("fullname", default=dic["userid"]))
    return res


def get_zope_root():
    """Get zope root from nowhere.
    Can be useful in IProcessStarting subscriber...
    """
    db = Zope2.DB
    connection = db.open()
    return connection.root().get("Application", None)


def set_site_from_package_config(product_name, zope_app=None):
    """Set site context from plone-path given a product-config zope config.
    Can be used in an IProcessStarting subscriber...

    <product-config XXX>
        plone-path YYY
    </product-config>

    :param product_name: configured product name string
    :param zope_app: zope application (to avoid to open a second connection if not necessary)
    :return: portal object if defined or None
    """
    config = getattr(getConfiguration(), "product_config", {})
    package_config = config.get(product_name)
    if package_config and package_config.get("plone-path"):  # can be set on instance1 only or not at all
        if not zope_app:
            zope_app = get_zope_root()
        try:
            site = zope_app.unrestrictedTraverse(package_config["plone-path"])
        except KeyError:  # site not found
            return None
        try:
            from zope.app.component.hooks import setSite
        except ImportError:
            from zope.component.hooks import setSite
        setSite(site)
        return site
    return None


def counted_logger(logger_name=""):
    """ """
    caller = inspect.stack()[1][3]
    logger = logging.getLogger("{0} ({1})".format(logger_name, inspect.stack()[1][3]))
    request = getRequest()
    req_key = "counter_logger__{0}".format(caller.replace(" ", ""))
    counter = request.get(req_key, 0) + 1
    request.set(req_key, counter)
    logger.info("Counter {0}".format(counter))
    return logger


def check_zope_admin():
    """
    Check if current user is the Zope admin.
    """
    user = getSecurityManager().getUser()
    if user.has_role("Manager") and user.__module__ in (
        "Products.PluggableAuthService.PropertiedUser",
        "AccessControl.users",
        "AccessControl.User",
    ):
        return True
    return False



# -*- coding: utf-8 -*-

from collective.MockMailHost.MockMailHost import MockMailHost
from imio.helpers.emailer import add_attachment
from imio.helpers.emailer import create_html_email
from imio.helpers.emailer import get_mail_host
from imio.helpers.emailer import InvalidEmailAddress
from imio.helpers.emailer import InvalidEmailAddressCharacters
from imio.helpers.emailer import InvalidEmailAddressFormat
from imio.helpers.emailer import send_email
from imio.helpers.emailer import validate_email_address
from imio.helpers.emailer import validate_email_addresses
from imio.helpers.testing import IntegrationTestCase

import os
import six


class TestEmail(IntegrationTestCase):
    def setUp(self):
        super(TestEmail, self).setUp()

    def test_create_html_email(self):
        msg = '<h1>Test</h1>\n<p><a href="https://github.com/">Github site</p>'
        eml = create_html_email(msg)
        estr = eml.as_string()
        self.assertIn("Content-Type: multipart/mixed;", estr)
        self.assertIn("Content-Type: multipart/alternative;", estr)
        self.assertIn('Content-Type: text/plain; charset="utf-8"', estr)
        if six.PY2:
            self.assertIn("Test =\n\n  Github site", estr)
        elif six.PY3:
            self.assertIn("Test=20\n  Github site", estr)
        self.assertIn('Content-Type: text/html; charset="utf-8"', estr)
        self.assertIn('<h1>Test</h1>\n<p><a href=3D"https://github.com/">Github site</p>', estr)

    def test_add_attachment(self):
        msg = '<h1>Test</h1>\n<p><a href="https://github.com/">Github site</p>'
        eml = create_html_email(msg)
        path = os.path.dirname(__file__)
        if six.PY3:
            barcode_resource = "barcode_python3_zint_60x60_156bytes.png"
        else:
            barcode_resource = "barcode_python2_zint_60x60_278bytes.png"
        filepath = os.path.join(path, barcode_resource)
        add_attachment(eml, "barcode.png", filepath=filepath)
        estr = eml.as_string()
        self.assertIn("Content-Type: multipart/mixed;", estr)
        self.assertIn("Content-Type: multipart/alternative;", estr)
        self.assertIn('Content-Type: text/plain; charset="utf-8"', estr)
        if six.PY2:
            self.assertIn("Test =\n\n  Github site", estr)
        elif six.PY3:
            self.assertIn("Test=20\n  Github site", estr)
        self.assertIn('Content-Type: text/html; charset="utf-8"', estr)
        self.assertIn('<h1>Test</h1>\n<p><a href=3D"https://github.com/">Github site</p>', estr)
        self.assertIn("Content-Type: application/octet-stream", estr)
        self.assertIn("Content-Transfer-Encoding: base64", estr)
        self.assertIn('Content-Disposition: attachment; filename="barcode.png"', estr)

    def test_send_email(self):
        msg = '<h1>Test</h1>\n<p><a href="https://github.com/">Github site</p>'
        eml = create_html_email(msg)
        path = os.path.dirname(__file__)
        if six.PY3:
            barcode_resource = "barcode_python3_zint_60x60_156bytes.png"
        else:
            barcode_resource = "barcode_python2_zint_60x60_278bytes.png"
        filepath = os.path.join(path, barcode_resource)
        add_attachment(eml, "barcode.png", filepath=filepath)
        mail_host = get_mail_host()
        mail_host.reset()

        call_args = []
        orig_mmh_send = MockMailHost.send

        def mock_send(*args, **kwargs):
            call_args.append(args)
            orig_mmh_send(*args, **kwargs)

        MockMailHost.send = mock_send
        if six.PY3:
            # Python 3 raises an error with accented characters in emails
            # see https://github.com/zopefoundation/Products.MailHost/issues/29
            # and https://stackoverflow.com/questions/52133735/how-do-i-send-email-to-addresses-with-non-ascii-
            #     characters-in-python
            send_email(eml, "Email subject hé hé", "noreply@from.org", "dest@to.org")
            self.assertIn(b"Subject: =?utf-8?q?Email_subject_h=C3=A9_h=C3=A9?=", mail_host.messages[0])
            self.assertIn(b"From: noreply@from.org", mail_host.messages[0])
            self.assertIn(b"To: dest@to.org", mail_host.messages[0])
            self.assertListEqual(call_args[0][2], ["dest@to.org"])
        else:
            send_email(eml, "Email subject hé hé", "noréply@from.org", "dèst@to.org")
            self.assertIn("Subject: =?utf-8?q?Email_subject_h=C3=A9_h=C3=A9?=\n", mail_host.messages[0])
            self.assertIn("From: nor\xc3\xa9ply@from.org\n", mail_host.messages[0])
            # self.assertIn('To: d\xc3\xa8st@to.org\n', mail_host.messages[0])
            self.assertIn("To: =?utf-8?q?d=C3=A8st=40to=2Eorg?=\n", mail_host.messages[0])
            self.assertListEqual(call_args[0][2], ["dèst@to.org"])
        mail_host.reset()
        call_args = []
        # multiple recipients
        send_email(eml, u"Email subject", "<noreply@from.org>", ["dest@to.org", "Stéphan Geulette <seg@to.org>"])
        self.assertListEqual(call_args[0][2], ["dest@to.org", "=?utf-8?q?St=C3=A9phan_Geulette?= <seg@to.org>"])
        if six.PY3:
            self.assertIn(b"To: dest@to.org, =?utf-8?q?St=C3=A9phan_Geulette?= <seg@to.org>", mail_host.messages[0])
        else:
            # self.assertIn('To: dest@to.org, =?utf-8?q?St=C3=A9phan_Geulette?= <seg@to.org>\n', mail_host.messages[0])
            self.assertIn(
                "To: =?utf-8?q?dest=40to=2Eorg=2C_=3D=3Futf-8=3Fq=3FSt=3DC3=3DA9phan=5FGeulett?=\n",
                mail_host.messages[0],
            )
        mail_host.reset()
        call_args = []
        # unicode parameters
        if six.PY3:
            self.assertTrue(send_email(eml, u"Email subject hé hé", u"noreply@from.org", u"dest@to.org"))
            self.assertTrue(
                send_email(
                    eml, u"Email subject", "<noreply@from.org>", ["dest@to.org", u"Stéphan Geulette <seg@to.org>"]
                )
            )
        else:
            # unicode is ok if singles
            self.assertTrue(send_email(eml, u"Email subject hé hé", u"noréply@from.org", u"dèst@to.org"))
            # not ok if in list
            self.assertRaises(
                UnicodeEncodeError,
                send_email,
                eml,
                u"Email subject",
                "<noreply@from.org>",
                ["dest@to.org", u"Stéphan Geulette <seg@to.org>"],
            )
        # cc, bcc and reply_to
        mail_host.reset()
        call_args = []
        send_email(
            eml,
            "Email subject",
            "noreply@from.org",
            "dest@to.org",
            mcc="copy@to.org",
            mbcc="bcc@to.org",
            replyto="reply@to.org",
        )
        # all recipients are in mto parameter
        self.assertListEqual(call_args[0][2], ["dest@to.org", "copy@to.org", "bcc@to.org"])
        if six.PY3:
            self.assertIn(b"From: noreply@from.org", mail_host.messages[0])
            self.assertIn(b"To: dest@to.org", mail_host.messages[0])
            self.assertIn(b"Cc: =?utf-8?q?copy=40to=2Eorg?=", mail_host.messages[0])
            self.assertIn(b"reply-to: =?utf-8?q?reply=40to=2Eorg?=", mail_host.messages[0])
            self.assertNotIn(b"=?utf-8?q?bcc=40to=2Eorg?=", mail_host.messages[0])
        else:
            self.assertIn("From: noreply@from.org", mail_host.messages[0])
            self.assertIn("To: =?utf-8?q?dest=40to=2Eorg?=\n", mail_host.messages[0])
            self.assertIn("Cc: =?utf-8?q?copy=40to=2Eorg?=\n", mail_host.messages[0])
            self.assertIn("reply-to: =?utf-8?q?reply=40to=2Eorg?=\n", mail_host.messages[0])
            self.assertNotIn("=?utf-8?q?bcc=40to=2Eorg?=\n", mail_host.messages[0])

        MockMailHost.send = orig_mmh_send

    def test_validate_email_address(self):
        self.assertTupleEqual(validate_email_address("name@domain.org"), (u"", u"name@domain.org"))
        self.assertTupleEqual(validate_email_address(u"name@domain.org"), (u"", u"name@domain.org"))
        self.assertTupleEqual(
            validate_email_address('"Real Name" <name@domain.org>'), (u"Real Name", u"name@domain.org")
        )
        self.assertTupleEqual(
            validate_email_address('"\\"Real Name\\"" <name@domain.org>'), (u'"Real Name"', u"name@domain.org")
        )
        self.assertTupleEqual(
            validate_email_address('""Real Name"" <name@domain.org>'), (u"Real Name", u"name@domain.org")
        )
        self.assertTupleEqual(validate_email_address("Real Name <name@domain.org>"), (u"Real Name", u"name@domain.org"))
        self.assertTupleEqual(validate_email_address("name@domain.org (Real Name)"), (u"Real Name", u"name@domain.org"))
        # errors on format
        self.assertRaises(InvalidEmailAddressFormat, validate_email_address, "<>")
        self.assertRaises(InvalidEmailAddressFormat, validate_email_address, "()")
        self.assertRaises(InvalidEmailAddressFormat, validate_email_address, "<,name@domain.org>")
        self.assertRaises(InvalidEmailAddressFormat, validate_email_address, "(name@domain.org)")
        # errors on email
        self.assertRaises(InvalidEmailAddress, validate_email_address, "[name@domain.org]")
        self.assertRaises(InvalidEmailAddress, validate_email_address, ",name@domain.org")
        self.assertRaises(InvalidEmailAddress, validate_email_address, "Real Name <na me@domain.org>")
        self.assertRaises(InvalidEmailAddress, validate_email_address, "na me@domain.org")
        self.assertRaises(InvalidEmailAddress, validate_email_address, "name|domain.org")
        self.assertRaises(InvalidEmailAddress, validate_email_address, "name@domainorg")
        self.assertRaises(InvalidEmailAddress, validate_email_address, "na<me@domain.org")
        self.assertRaises(InvalidEmailAddress, validate_email_address, "name@domain.or(g")
        self.assertRaises(InvalidEmailAddress, validate_email_address, "name@domain.or)g")
        # errors on realname
        self.assertRaises(InvalidEmailAddressCharacters, validate_email_address, "Stéphan <stephan@domain.org>")

    def test_validate_email_addresses(self):
        self.assertListEqual(validate_email_addresses("name@domain.org"), [(u"", u"name@domain.org")])
        self.assertListEqual(validate_email_addresses(u"name@domain.org"), [(u"", u"name@domain.org")])
        self.assertListEqual(
            validate_email_addresses('"Real Name" <name@domain.org>'), [(u"Real Name", u"name@domain.org")]
        )
        self.assertListEqual(
            validate_email_addresses("Real Name <name@domain.org>"), [(u"Real Name", u"name@domain.org")]
        )
        self.assertListEqual(
            validate_email_addresses("name@domain.org (Real Name)"), [(u"Real Name", u"name@domain.org")]
        )
        self.assertListEqual(
            validate_email_addresses("name1@domain.org, name2@domain.org"),
            [(u"", u"name1@domain.org"), (u"", u"name2@domain.org")],
        )
        self.assertListEqual(
            validate_email_addresses("Real Name <name1@domain.org>, name2@domain.org"),
            [(u"Real Name", u"name1@domain.org"), (u"", u"name2@domain.org")],
        )
        self.assertListEqual(
            validate_email_addresses('Real Name <name1@domain.org>, "Other Name" <name2@domain.org>'),
            [(u"Real Name", u"name1@domain.org"), (u"Other Name", u"name2@domain.org")],
        )
        self.assertListEqual(
            validate_email_addresses('"Real, Name" <name@domain.org>'), [(u"Real, Name", u"name@domain.org")]
        )
        self.assertListEqual(
            validate_email_addresses('"Real, Name" <name1@domain.org>, ' '"Other, Name" <name2@domain.org>'),
            [(u"Real, Name", u"name1@domain.org"), (u"Other, Name", u"name2@domain.org")],
        )
        # errors on format
        self.assertRaises(InvalidEmailAddressFormat, validate_email_addresses, "<>")
        self.assertRaises(InvalidEmailAddressFormat, validate_email_addresses, "()")
        self.assertRaises(InvalidEmailAddressFormat, validate_email_addresses, "(name@domain.org)")
        # errors on email
        self.assertRaises(InvalidEmailAddress, validate_email_addresses, "[name@domain.org]")
        self.assertRaises(InvalidEmailAddress, validate_email_addresses, "Real Name <na me@domain.org>")
        self.assertRaises(InvalidEmailAddress, validate_email_addresses, "na me@domain.org")
        self.assertRaises(InvalidEmailAddress, validate_email_addresses, "Real, Name <name@domain.org>")
        self.assertRaises(InvalidEmailAddress, validate_email_addresses, "name|domain.org")
        self.assertRaises(InvalidEmailAddress, validate_email_addresses, "name@domainorg")
        self.assertRaises(InvalidEmailAddress, validate_email_addresses, "na<me@domain.org")
        self.assertRaises(InvalidEmailAddress, validate_email_addresses, "name@domain.or(g")
        self.assertRaises(InvalidEmailAddress, validate_email_addresses, "name@domain.or)g")
        # errors on realname
        self.assertRaises(InvalidEmailAddressCharacters, validate_email_addresses, "Stéphan <stephan@domain.org>")



# -*- coding: utf-8 -*-
"""Base module for unittesting."""
from imio.helpers.cache import setup_ram_cache
from imio.helpers.ram import imio_global_cache
from imio.helpers.ram import IMIORAMCache
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import login
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.testing import z2
from zope.component import getSiteManager
from zope.component import getUtility
from zope.ramcache.interfaces.ram import IRAMCache

import imio.helpers
import logging
import sys
import unittest


def testing_logger(logger_name=""):
    logger = logging.getLogger(logger_name)
    logger.setLevel(logging.DEBUG)
    handler = logging.StreamHandler(sys.stdout)
    formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    return logger


class PloneWithHelpersLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        """Set up Zope."""
        # Load ZCML
        self.loadZCML(package=imio.helpers, name="testing.zcml")

    def setUpPloneSite(self, portal):
        """Set up Plone."""
        # Install into Plone site using portal_setup
        applyProfile(portal, "imio.helpers:testing")
        applyProfile(portal, "collective.MockMailHost:default")

        # use intranet_workflow for every types
        portal.portal_workflow.setDefaultChain("intranet_workflow")
        # Login and create some test content
        setRoles(portal, TEST_USER_ID, ["Manager"])
        login(portal, TEST_USER_NAME)
        folder_id = portal.invokeFactory("Folder", "folder")
        portal[folder_id].reindexObject()
        folder2_id = portal.invokeFactory("Folder", "folder2")
        portal[folder2_id].reindexObject()

        if not isinstance(getUtility(IRAMCache), IMIORAMCache):
            sml = getSiteManager(portal)
            sml.unregisterUtility(provided=IRAMCache)
            sml.registerUtility(component=imio_global_cache, provided=IRAMCache)
            setup_ram_cache()

        # Commit so that the test browser sees these objects
        import transaction

        transaction.commit()

    def tearDownZope(self, app):
        """Tear down Zope."""
        z2.uninstallProduct(app, "imio.helpers")


FIXTURE = PloneWithHelpersLayer(name="FIXTURE")


INTEGRATION = IntegrationTesting(bases=(FIXTURE,), name="INTEGRATION")


FUNCTIONAL = FunctionalTesting(bases=(FIXTURE,), name="FUNCTIONAL")


class CommonTestCase(unittest.TestCase):
    def setUp(self):
        super(CommonTestCase, self).setUp()
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        self.request = self.portal.REQUEST
        self.catalog = self.portal.portal_catalog


class IntegrationTestCase(CommonTestCase):
    """Base class for integration tests."""

    layer = INTEGRATION


class FunctionalTestCase(CommonTestCase):
    """Base class for functional tests."""

    layer = FUNCTIONAL
